@@ -1,6 +1,5 @@
-from Acquisition import aq_parent, aq_base, aq_inner
+from Acquisition import aq_base
 from Products.Five import BrowserView
-from Products.Five.site.localsite import enableLocalSiteHook, disableLocalSiteHook
 from zope.app.intid.interfaces import IIntIds
 from zope.app.component.hooks import setSite, setHooks
 from zope.app.component.interfaces import ISite
@@ -9,6 +8,7 @@ from zope.component import getUtility, getSiteManager
 from OFS.interfaces import IApplication
 from intid import IntIds, OFSIntIds
 from lsm import make_site, USE_LSM
+from utils import aq_iter
 
 class FiveIntIdsInstall(BrowserView):
     @property
@@ -23,15 +23,20 @@ class FiveIntIdsInstall(BrowserView):
             self.install()
 
     def install(self):
-        addUtility(self.context, IIntIds, findroot=False)
+        add_intids(self.context, findroot=False)
 
     @property
     def installed(self):
         installed = False
         try:
             intids = getUtility(IIntIds)
-            if intids:
-                installed = True
+            if intids is not None:
+                if USE_LSM:
+                    sm = self.context.getSiteManager()
+                    if 'intids' in sm.objectIds():
+                        installed = True
+                else:
+                    installed = True
         except ComponentLookupError, e:
             pass
         return installed
@@ -43,24 +48,12 @@ def initializeSite(site, sethook=False, **kw):
     setSite(site)
 
 def get_root(app):
-    # adapted from alecm's 'listen'
-    seen = {}
-    # get the inner-most wrapper (maybe save some cycles, and prevent
-    # bogus loop detection)
-    app = aq_inner(app)
-    while app is not None and not IApplication.providedBy(app):
-        seen[id(aq_base(app))] = 1
-        app = getattr(app, 'aq_parent', getattr(app, '__parent__', None))
-        if id(aq_base(app)) in seen:
-            # avoid loops resulting from acquisition-less views
-            # whose __parent__ points to
-            # the context whose aq_parent points to the view
-            raise AttributeError, '__parent__ loop found'
-    if app is None:
-        raise AttributeError, 'No application found'
-    return app
+    for parent in aq_iter(app, error=AttributeError):
+        if IApplication.providedBy(parent):
+            return parent
+    raise AttributeError, 'No application found'
 
-def addUtility(site, interface, klass, name='', findroot=True):
+def addUtility(site, interface, klass, name='', ofs_name='', findroot=True):
     """
     add local utility in zope2
     """
@@ -70,23 +63,42 @@ def addUtility(site, interface, klass, name='', findroot=True):
 
     # If we have the zope Application and the utility is not yet
     # registered, then register it.
-    assert app, TypeError("app is None")
+    assert app is not None, TypeError("app is None")
 
     if not ISite.providedBy(app):
         initializeSite(app, sethook=False)
 
     sm = app.getSiteManager()
+    obj = None
+    if USE_LSM:
+        # Try to get the utility from OFS directly in case it is
+        # stored, but not registered
+        ofs_name = ofs_name or name
+        obj = getattr(aq_base(sm), ofs_name, None)
     if sm.queryUtility(interface,
                        name=name,
                        default=None) is None:
-
-        if name: obj = klass(name)
-        else: obj = klass()
+        # Register the utility if it is not yet registered
+        if obj is None:
+            if name:
+                obj = klass(name)
+            else:
+                obj = klass()
         if USE_LSM:
             sm.registerUtility(provided=interface, component=obj,
                                name=name)
         else:
             sm.registerUtility(interface, obj, name=name)
+    elif USE_LSM and obj is None:
+        # Get the utility if registered, but not yet stored in the LSM
+        obj = sm.queryUtility(interface, name=name)
+
+    # Make sure we store the utility permanently in the OFS so we don't loose
+    # intids on uninstall
+    if USE_LSM:
+        if ofs_name and ofs_name not in sm.objectIds():
+            sm._setObject(ofs_name, aq_base(obj), set_owner=False,
+                          suppress_events=True)
 
 from intid import IIntIds, OFSIntIds
 from zope.component import getUtility
@@ -96,7 +108,7 @@ def add_intids(site, findroot=False):
         klass = IntIds
     else:
         klass = OFSIntIds
-    addUtility(site, IIntIds, klass, findroot=findroot)
+    addUtility(site, IIntIds, klass, ofs_name='intids', findroot=findroot)
 
 def get_intids(context=None):
     return getUtility(IIntIds, context=context)
